import PyIPSDK
import PyIPSDK.IPSDKIPLArithmetic as arithm
import PyIPSDK.IPSDKIPLBinarization as bin
import PyIPSDK.IPSDKIPLGlobalMeasure as glbmsr
import PyIPSDK.IPSDKIPLIntensityTransform as itrans
import PyIPSDK.IPSDKIPLLogical as logic
import PyIPSDK.IPSDKIPLUtility as util

import numpy as np

def blendOverlay(im, ov, alpha):
	
	# The user can define the color of the n first labels (optional)
	# In the example, the first 3 labels are blue, green and red
	colorUser = {}
	colorUser[1]= [0,0,255]
	colorUser[2]= [0,255,0]
	colorUser[3]= [255,0,0]
	
	# Convert the input image if necessary
	im_UInt8 = im
	if(im.getBufferType() != PyIPSDK.eIBT_UInt8):
		range_UInt8 = PyIPSDK.createRange(0, 255)
		im_normalized = itrans.normalizeImg(im, range_UInt8)
		im_UInt8 = util.convertImg(im_normalized, PyIPSDK.eIBT_UInt8)

	# Create the ouput image geometry
	geometry = None
	bIs2d = im.getVolumeGeometryType() == PyIPSDK.eVGT_2d
	if bIs2d:
		geometry = PyIPSDK.geometryRgb2d(PyIPSDK.eIBT_UInt8, im.getSizeX(), im.getSizeY())
	else:
		geometry = PyIPSDK.geometryRgb3d(PyIPSDK.eIBT_UInt8, im.getSizeX(), im.getSizeY(), im.getSizeZ())

	# Create the output color image
	imColor = None
	if im.getColorGeometryType() == PyIPSDK.eCGT_Grey :
		imColor = PyIPSDK.createImage(geometry)

		if bIs2d:
			# Copy the channels
			for c in range(0, 3):
				plan = PyIPSDK.extractPlan(0, c, 0, imColor)
				util.copyImg(im_UInt8, plan)
		else:
			# Copy the channels
			for c in range(0, 3):
				plan = PyIPSDK.extractVolume(c, 0, imColor)
				util.copyImg(im_UInt8, plan)
	else:
		imColor = im_UInt8

	# Count the number of labels
	statsRes = None
	if bIs2d:
		statsRes = glbmsr.statsMsr2d(ov)
	else:
		statsRes = glbmsr.statsMsr3d(ov)
	nbLabels = statsRes.max

	# Create 3 random LUTs (one per channel)
	randValues = np.random.rand(3, int(nbLabels+1))*255
	for i in colorUser:
		if i <=nbLabels:
			for c in range(3):
				randValues[c][i] = colorUser[i][c]

	lutR = PyIPSDK.createIntensityLUT(0, 1, randValues[0, :])
	lutG = PyIPSDK.createIntensityLUT(0, 1, randValues[1, :])
	lutB = PyIPSDK.createIntensityLUT(0, 1, randValues[2, :])
	colorLut = [lutR, lutG, lutB]

	# Convert the label image to a color image
	overlayImage = PyIPSDK.createImage(imColor.getGeometry())
	if bIs2d:
		for c in range(0, 3):
			plan = PyIPSDK.extractPlan(0, c, 0, overlayImage)
			itrans.lutTransform2dImg(ov, colorLut[c], plan)
	else:
		for c in range(0, 3):
			plan = PyIPSDK.extractVolume(c, 0, overlayImage)
			itrans.lutTransform3dImg(ov, colorLut[c], plan)

	# Blending
	blend = arithm.blendImgImg(imColor, overlayImage, alpha)
	blend = itrans.normalizeImg(blend, PyIPSDK.createRange(0, 255))
	blend = util.convertImg(blend, PyIPSDK.eIBT_UInt8)

	mask = bin.lightThresholdImg(ov, 1)

	maskImage = None
	binaryGeometry = None
	bIs2d = im.getVolumeGeometryType() == PyIPSDK.eVGT_2d
	if bIs2d:
		binaryGeometry = PyIPSDK.geometryRgb2d(PyIPSDK.eIBT_Binary, im.getSizeX(), im.getSizeY())
	else:
		binaryGeometry = PyIPSDK.geometryRgb3d(PyIPSDK.eIBT_Binary, im.getSizeX(), im.getSizeY(), im.getSizeZ())
	maskImage = PyIPSDK.createImage(binaryGeometry)

	if bIs2d:
		for c in range(0, 3):
			plan = PyIPSDK.extractPlan(0, c, 0, maskImage)
			util.copyImg(mask, plan)
	else:
		for c in range(0, 3):
			plan = PyIPSDK.extractVolume(c, 0, maskImage)
			util.copyImg(mask, plan)
	logic.maskImgImg(blend, imColor, maskImage, blend)

	return blend

























